@@ -1,5 +1,6 @@
 #!/bin/bash
 #%stage: softraid
+# grep needed for udev rules file.
 #%programs: /sbin/mdadm /sbin/mdmon
 #%modules: raid0 raid1 raid10 raid456
 #%if: -n "$need_mdadm"
@@ -17,11 +18,8 @@
 # load the necessary module before we initialize the raid system
 load_modules
 
-# put the mdmon socked and pid file to /dev/.mdadm
-rm -rf /var/run/mdadm
-mkdir -p /var/run
-ln -s /dev/.mdadm /var/run/mdadm
-mkdir -p /dev/.mdadm
+# Create dir for  socket and pid files
+mkdir -p /run/mdadm
 [ "$mduuid" ] && md_uuid="$mduuid"
 
 md_major=$(sed -ne 's/\s*\([0-9]\+\)\s*md$/\1/p' /proc/devices)
@@ -104,31 +102,34 @@ md_assemble()
 	"")
 		;;
 	/dev/*)
-		$mdadm -A $mdconf $container
+		$mdadm -A $mdconf $container --offroot
 		;;
 	[0-9a-f]*[0-9a-f])
 		container_name=$(get_md_name "$container")
 		if test -z "$container_name"; then
 			container_name=/dev/md/container
 		fi
-		$mdadm -A $mdconf --uuid="$container" "$container_name"
+		$mdadm -A $mdconf --uuid="$container" "$container_name" --offroot
 		;;
 	*)
 		echo "unrecognized container for $dev: $container"
 	esac
 	if test -n "$dev"; then
-		$mdadm -A $mdconf "$dev"
+		$mdadm -A $mdconf "$dev" --offroot
 	else
 		dev=$(get_md_name "$uuid")
 		if test -z "$dev"; then
 			# mdadm will pick a device name
-			$mdadm -A $mdconf --uuid=$uuid
+			$mdadm -A $mdconf --uuid=$uuid --offroot
 		else
-			$mdadm -A $mdconf --uuid=$uuid "$dev"
+			$mdadm -A $mdconf --uuid=$uuid "$dev" --offroot
 		fi
 	fi
 }
 
+# run any degraded arrays assembled incrementally
+wait_for_events
+$mdadm --incremental --run --scan --offroot
 md_assemble "$resumedev"
 md_assemble "$rootdev"
 if [ -n "$md_dev" ] ; then
