@@ -54,11 +54,11 @@ sub import {
     return;
 }
 
-die "Perl lib version (5.16.0) doesn't match executable '$0' version ($])"
+die "Perl lib version (5.16.2) doesn't match executable '$0' version ($])"
     unless $^V;
 
-$^V eq 5.16.0
-    or die "Perl lib version (5.16.0) doesn't match executable '$0' version (" .
+$^V eq 5.16.2
+    or die "Perl lib version (5.16.2) doesn't match executable '$0' version (" .
 	sprintf("v%vd",$^V) . ")";
 
 
@@ -83,7 +83,7 @@ sub AUTOLOAD {
 
 # tie returns the object, so the value returned to require will be true.
 tie %Config, 'Config', {
-    archlibexp => '/usr/lib/perl5/5.16.0/i586-linux-thread-multi',
+    archlibexp => '/usr/lib/perl5/5.16.2/i586-linux-thread-multi',
     archname => 'i586-linux-thread-multi',
     cc => 'cc',
     d_readlink => 'define',
@@ -97,14 +97,14 @@ tie %Config, 'Config', {
     ldlibpthname => 'LD_LIBRARY_PATH',
     libpth => '/lib /usr/lib /usr/local/lib',
     osname => 'linux',
-    osvers => '3.4.6-2.10-xen',
+    osvers => '3.4.6-2.10-default',
     path_sep => ':',
-    privlibexp => '/usr/lib/perl5/5.16.0',
+    privlibexp => '/usr/lib/perl5/5.16.2',
     scriptdir => '/usr/bin',
-    sitearchexp => '/usr/lib/perl5/site_perl/5.16.0/i586-linux-thread-multi',
-    sitelibexp => '/usr/lib/perl5/site_perl/5.16.0',
+    sitearchexp => '/usr/lib/perl5/site_perl/5.16.2/i586-linux-thread-multi',
+    sitelibexp => '/usr/lib/perl5/site_perl/5.16.2',
     so => 'so',
     useithreads => 'define',
     usevendorprefix => 'define',
-    version => '5.16.0',
+    version => '5.16.2',
 };
