@@ -82,7 +82,7 @@ use base 'Exporter';
 
 our @EXPORT = qw(InitLibrary CountImageSections CountSections
 		 RemoveImageSections GetDefaultImage
-		 GetDefaultInitrd GetBootloader UpdateBootloader GetFittingSection
+		 GetDefaultInitrd GetBootloader GetSecureBoot UpdateBootloader GetFittingSection
 		 GetGlobals SetGlobals 
 		 GetSectionList GetSection 
 		 AddSection RemoveSections
@@ -91,7 +91,9 @@ our @EXPORT = qw(InitLibrary CountImageSections CountSections
 use Bootloader::Library;
 use Bootloader::Core;
 use Bootloader::Path;
-use Bootloader::Logger;
+
+use Cwd;
+use File::Find;
 
 my $lib_ref = undef;
 my $dmsetup = undef;
@@ -99,87 +101,21 @@ my $mdadm = undef;
 my $multipath = undef;
 my $devicemapper = undef;
 
-sub DumpLog {
-    my $core_lib = shift;
 
-    my $using_logfile = 1;
-    my $logname = Bootloader::Path::Logname();
+# dummy compat function
+sub DumpLog { }
 
-    if (not open LOGFILE, ">>$logname") {
-	$using_logfile = 0;
-	open LOGFILE, ">&STDERR" or die "Can't dup STDERR: $!";
-	print LOGFILE ("WARNING: Can't open $logname, using STDERR instead.\n");
-    }
-
-    # Adding timestamp to log messages
-    use POSIX qw(strftime);
-
-    sub timestamp () {
-	return strftime ( "%Y-%m-%d %H:%M:%S", localtime);
-    }
-
-    foreach my $rec (@{Bootloader::Logger::instance()->GetLogRecords()})
-    {
-	my $message = $rec->{"message"};
-	my $level = $rec->{"level"};
-
-	# If debug messages should be printed, the environment variable
-	# Y2DEBUG has to be set ("export Y2DEBUG=1").
-	if ($level eq "debug" and defined $ENV{'Y2DEBUG'})
-	{
-	    print LOGFILE (timestamp() . " DEBUG: $message\n");
-	}
-	elsif ($level eq "debug" and not defined $ENV{'Y2DEBUG'})
-	{
-	    # Omit debug messages
-	}
-	elsif ($level eq "milestone")
-	{
-	    print LOGFILE (timestamp() . " MILESTONE: $message\n");
-	}
-	elsif ($level eq "warning")
-	{
-	    print LOGFILE (timestamp() . " WARNING: $message\n");
-
-	    # If writing to perl logfile, also print warnings to STDERR
-	    if ($using_logfile) {
-		print STDERR ("Perl-Bootloader: ".timestamp() . " WARNING: $message\n");
-	    }
-	}
-	elsif ($level eq "error")
-	{
-	    print LOGFILE (timestamp() . " ERROR: $message\n");
-
-	    # If writing to perl logfile, also print errors to STDERR
-	    if ($using_logfile) {
-		print STDERR ("Perl-Bootloader: ".timestamp() . " ERROR: $message\n");
-	    }
-	}
-	else
-	{
-	    print LOGFILE (timestamp() . " ERROR: Uncomplete log record\n");
-	    print LOGFILE (timestamp() . " ERROR: $message\n");
-
-	    # If writing to perl logfile, also print errors to STDERR
-	    if ($using_logfile) {
-		print STDERR (timestamp() . " ERROR: Uncomplete log record\n");
-		print STDERR (timestamp() . " ERROR: $message\n");
-	    }
-	}
-    }
-    close LOGFILE;
-}
 
 sub ResolveCrossDeviceSymlinks {
-    my $path = shift;
+  my $path = shift;
 
-    my $core_lib = Bootloader::Core->new ();
-    $path = $core_lib->ResolveCrossDeviceSymlinks ($path);
+  my $core_lib = Bootloader::Core->new($lib_ref);
+  $path = $core_lib->ResolveCrossDeviceSymlinks($path);
 
-    DumpLog ($core_lib);
-    return $path;
+  return $path;
 }
 
+
 =item
 C<< $mp_ref = Bootloader::Tools::ReadMountPoints (); >>
 
@@ -190,35 +126,31 @@ See InitLibrary function for example.
 
 =cut
 
-sub ReadMountPoints {
-    my $udevmap = shift;
-    open (FILE, Bootloader::Path::Fstab()) || 
-	die ("ReadMountPoints(): Failed to open /etc/fstab");
+sub ReadMountPoints
+{
+  my $self = $lib_ref;
 
-    my %mountpoints = ();
-    while (my $line = <FILE>)
-    {
-	if ($line =~ /^\s*(\S+)\s+(\S+).*/)
-	{
-	    my $dev = $1;
-	    my $mp = $2;
-	    if (substr ($dev, 0, 1) ne "#")
-	    {
-		if ($dev =~ m/^LABEL=(.*)/)
-		{
-                    $dev = "/dev/disk/by-label/$1"; #do not translate otherwise it changes root always bnc#575362
-                } elsif ($dev =~ m/UUID=(.*)/){
-                    $dev = "/dev/disk/by-uuid/$1";
-                }
-                $mp =~ s/\\040/ /; #handle spaces in fstab
-		$mountpoints{$mp} = $dev;
-	    }
-	}
+  my $udevmap = shift;
+
+  my %mountpoints = ();
+
+  for my $line (@{$self->ReadFile(Bootloader::Path::Fstab())}) {
+    (my $dev, my $mp) = split ' ', $line;
+    next if $dev =~ /^#/;
+    if($dev =~ m/^LABEL=(.*)/) {
+      $dev = "/dev/disk/by-label/$1";	# do not translate otherwise it changes root always bnc#575362
     }
-    close (FILE);
-    return \%mountpoints;
+    elsif($dev =~ m/^UUID=(.*)/) {
+      $dev = "/dev/disk/by-uuid/$1";
+    }
+    $mp =~ s/\\040/ /g;			# handle spaces in fstab
+    $mountpoints{$mp} = $dev;
+  }
+
+  return \%mountpoints;
 }
 
+
 =item
 C<< $part_ref = Bootloader::Tools::ReadPartitions (); >>
 
@@ -234,10 +166,9 @@ sub ReadPartitions {
     my $udevmap = shift;
     my $sb = "/sys/block";
     my $mounted = undef;
-    my $logger = Bootloader::Logger::instance();
     unless (-e $sb) {
       $mounted = `mount /sys`;
-      $logger->milestone("ReadPartitions: Mount /sys");
+      $lib_ref->milestone("Mount /sys");
     }
     opendir(BLOCK_DEVICES, "$sb") || 
 	die ("ReadPartitions(): Failed to open dir $sb");
@@ -248,7 +179,7 @@ sub ReadPartitions {
     } readdir(BLOCK_DEVICES);
     closedir BLOCK_DEVICES;
 
-    $logger->milestone("ReadPartitions: Found disks: ". join (",",@disks));
+    $lib_ref->milestone("disks =", \@disks);
 
     # get partition info for all partitions on all @disks
     my @devices = ();
@@ -273,7 +204,7 @@ sub ReadPartitions {
 	    } readdir (BLOCK_DEVICES);
 	    closedir BLOCK_DEVICES;
 
-            $logger->milestone("ReadPartitions: Found parts: ". join (",",@parts));
+            $lib_ref->milestone("partitions =", \@parts);
 
 	    # generate proper device names and other info for all @part[ition]s
       #raid have ! in names for /dev/raid/name (bnc#607852)
@@ -325,7 +256,6 @@ Gets multipath configuration. Return reference to hash map, empty if system does
 
 sub GetMultipath {
   my %ret = ();
-  my $logger = Bootloader::Logger::instance();
 
   unless (DMRaidAvailable())
   {
@@ -340,9 +270,9 @@ sub GetMultipath {
     # return if problems occurs...typical is not loaded kernel module
     if ( $? ) {
       if ($result[0] =~ m/kernel driver not loaded/) {
-        $logger->milestone("Tools::GetMultipath: multipath kernel module is not loaded Ecode:$?.");
+        $lib_ref->milestone("multipath kernel module is not loaded Ecode:$?.");
       } else {
-        $logger->warning("Tools::GetMultipath: multipath command failed with $?.");
+        $lib_ref->warning("multipath command failed with $?.");
       }
       return \%ret;
     }
@@ -352,7 +282,7 @@ sub GetMultipath {
     my $line = "";
     $line = shift @result if (scalar @result != 0);
     while (scalar @result != 0){
-      $logger->milestone("Tools::GetMultipath: processing line $line");
+      $lib_ref->milestone("processing line $line");
       if ($line !~ m/^(\S+)\s.*dm-\d+.*$/){
         $line = shift @result;
         next;
@@ -361,39 +291,62 @@ sub GetMultipath {
       while (scalar @result != 0){
         $line = shift @result;
         chomp $line;
-        $logger->milestone("Tools::GetMultipath: processing line $line");
+        $lib_ref->milestone("processing line $line");
         if ($line =~ m/(.*)dm-.*$/){
           last;
         }
         if ($line =~ m/\d+:\d+:\d+:\d+\s+(\S+)\s+/){
           $ret{"/dev/$1"} = $multipathdev;
-          $logger->milestone("Tools::GetMultipath: added /dev/$1 -> $multipathdev");
+          $lib_ref->milestone("added /dev/$1 -> $multipathdev");
         }
       }
     }
   }
   else
   {
-    $logger->milestone("Tools::GetMultipath: multipath command not installed");
+    $lib_ref->milestone("multipath command not installed");
   }
 
   return \%ret;
 }
 
 =item
-C<< Bootloader::Tools::GetMultipath (); >>
+C<< Bootloader::Tools::GetDeviceMapping (); >>
 
-Gets multipath configuration. Return reference to hash map, empty if system doesn't contain multipath.
+Return reference to hash with symlinks to block devices.
 
 =cut
 
+# Notes:
+#   - replaces GetUdevMapping()
+#   - no logging as map is passed to DefineUdevMapping() and logged there
+#   - if issues with device mapper arise, check bnc #590637
+sub GetDeviceMapping
+{
+  my $list;
+
+  find({ wanted => sub {
+    if(-l && -b) {
+      my $b = $_;
+      $b =~ s#[^/]+$##;
+      my $l = readlink;
+      if(defined $l) {
+        $l = "$b$l" if $l !~ /^\//;
+        $list->{$_} = Cwd::realpath($l);
+      }
+    }
+  }, no_chdir => 1 } , "/dev");
+
+  return $list;
+}
+
+
 sub GetUdevMapping {
   my %mapping= ();
   local $_;
 
   my @output = `find -P /dev -type b`;
   chomp @output;
-  my $logger = Bootloader::Logger::instance();
   for my $dev (@output) {
     next if ($dev =~ m:^/dev/mapper/:);
 
@@ -416,7 +369,7 @@ sub GetUdevMapping {
         }
       }
 
-      $logger->error("UDEVMAPPING: dmdev $dev doesn't have defined DM_NAME in udev") unless defined $dmdev;
+      $lib_ref->error("UDEVMAPPING: dmdev $dev doesn't have defined DM_NAME in udev") unless defined $dmdev;
       my $prevdev = $dev;
       $dev = "/dev/mapper/$dmdev";
       # DM_NAME could contain also part in some case (bnc#590637)
@@ -429,27 +382,30 @@ sub GetUdevMapping {
     }
   }
 
-  while (my ($k,$v) = each (%mapping)){
-    $logger->milestone("UDEV MAPPING: ".($k||"")." -> ".($v||""));
-  }
+#  while (my ($k,$v) = each (%mapping)){
+#    $lib_ref->milestone("UDEV MAPPING: ".($k||"")." -> ".($v||""));
+#  }
 
   if(!(keys %mapping)) {
-    $logger->milestone("*** WARNING: No UDEV mapping! ***");
-    $logger->milestone("device tree:");
+    $lib_ref->milestone("*** WARNING: No UDEV mapping! ***");
+    $lib_ref->milestone("device tree:");
     for (`find -P /dev -type b -ls 2>/dev/null`) {
       chomp;
-      $logger->milestone("  $_");
+      $lib_ref->milestone("  $_");
     }
-    $logger->milestone("udevinfo:");
+    $lib_ref->milestone("udevinfo:");
     for (`udevadm info -e 2>&1`) {
       chomp;
-      $logger->milestone("  $_");
+      $lib_ref->milestone("  $_");
     }
   }
 
+  # $lib_ref->milestone("\%mapping =", \%mapping);
+
   return \%mapping;
 }
 
+
 =item
 C<< $numDM = Bootloader::Tools::DMRaidAvailable (); >>
 
@@ -458,36 +414,47 @@ Return 0 if no device, 1 if there are any.
 
 =cut
 
-sub DMRaidAvailable {
-    my $retval = 0;
+sub DMRaidAvailable
+{
+  my $self = $lib_ref;
 
-    my $logger = Bootloader::Logger::instance();
-    $logger->milestone(`cat /proc/misc`);
+  my $r = $self->{cache}{DMRaidAvailable};
 
-    # Check if device-mapper is available in /proc/misc
-    my $dm_available = qx{grep device-mapper /proc/misc};
-    chomp $dm_available;
+  if(defined $r) {
+    $self->milestone("DMRaidAvailable (cached) = $r");
+    return $r;
+  }
 
-    if ($dm_available eq "") {
-	return $retval;
-    }
+  $self->{tools}{dmsetup} = $dmsetup = AddPathToExecutable("dmsetup");
 
-    $dmsetup = AddPathToExecutable("dmsetup");
+  # check for device-mapper in /proc/misc
+  $r = grep /device-mapper/, @{$self->ReadFile(Bootloader::Path::Prefix('/proc/misc'))};
 
+  $self->milestone("device-mapper = $r");
+
+  if($r) {
+    # check if dmsetup works and reports dm devices
     if (-e $dmsetup) {
-	my $dm_devices = qx{$dmsetup info -c --noheadings -o uuid};
-	chomp($dm_devices);
+      my $dm_devices = qx{$dmsetup info -c --noheadings -o uuid 2>/dev/null};
+      chomp $dm_devices;
+      $r = $dm_devices eq "No devices found" || $dm_devices eq "" ? 0 : 1;
 
-	$retval = $dm_devices ne "No devices found";
+      $self->milestone("dm devices = $r");
     }
     else {
-	$logger->error("The command \"dmsetup\" is not available.");
-	$logger->error("Is the package \"device-mapper\" installed?");
+      $self->error('The command "dmsetup" is not available.');
+      $self->error('Is the package "device-mapper" installed?');
     }
-    
-    return $retval;
+  }
+
+  $r = $r ? 1 : 0;
+
+  $self->milestone("DMRaidAvailable = $r");
+
+  return $self->{cache}{DMRaidAvailable} = $r;
 }
 
+
 =item
 C<< $part_ref = Bootloader::Tools::ReadDMRaidPartitions (); >>
 
@@ -508,8 +475,6 @@ sub ReadDMRaidPartitions {
     my @dmdisks = ();
     my @dmparts = ();
 
-    my $logger = Bootloader::Logger::instance();
-
     open(DMDEV, "$dmsetup info -c --noheadings -o name |") || 
 	die ("ReadDMRaidPartitions(): dmsetup failed.");
 
@@ -519,14 +484,14 @@ sub ReadDMRaidPartitions {
 
 	#FIXME: I should not need to do this twice
 	if ($dmdev !~ m/part/) {
-            $logger->milestone("Find raid partition $dmdev");
+            $lib_ref->milestone("Find raid partition $dmdev");
 	    # $dmdev is the base device
 	    $dmdev = "/dev/mapper/" . $dmdev;
 	    push @dmdisks, $dmdev;
 	}
 	#FIXME: need to check what needs to be removed
 	else {
-            $logger->milestone("Find raid disk $dmdev");
+            $lib_ref->milestone("Find raid disk $dmdev");
 	    $dmdev = "/dev/mapper/" . $dmdev;
 	    push @dmparts, $dmdev;
 	}
@@ -660,7 +625,6 @@ to initialize the bootloader library properly.
 # FIXME: this has to be read through yast::storage
 sub ReadRAID1Arrays {
     my $udevmapping = shift;
-    my $logger = Bootloader::Logger::instance();
     my %mapping = ();
     # use '/sbin/mdadm --detail --verbose --scan'
     # Assuming an output format like:
@@ -671,14 +635,14 @@ sub ReadRAID1Arrays {
     #	    devices=/dev/sda1,/dev/sdb1
     #
 
-    $mdadm = AddPathToExecutable("mdadm");
+    $lib_ref->{tools}{mdadm} = $mdadm = AddPathToExecutable("mdadm");
 
     if (-e $mdadm) {
-      open (MD, "$mdadm --detail --verbose --scan |");
+      open (MD, "$mdadm --detail --verbose --scan 2>/dev/null |");
     }
     else {
-    	$logger->milestone ("The command \"mdadm\" is not available.");
-    	$logger->milestone ("Expect that system doesn't have MD array.");
+    	$lib_ref->milestone ('The command "mdadm" is not available.');
+    	$lib_ref->milestone ("Expect that system doesn't have MD array.");
 
     	# If the command "mdadm" isn't available, return a reference to an
     	# empty hash
@@ -686,31 +650,31 @@ sub ReadRAID1Arrays {
     }
 
     my ($array, $level, $num_devices);
-    $logger->milestone("Tools::ReadRAID1Arrays: start parsing mdadm --detail --verbose --scan:");
+    $lib_ref->milestone("start parsing mdadm --detail --verbose --scan");
     while (my $line = <MD>)
     {
         chomp ($line);
-        $logger->milestone("Tools::ReadRAID1Arrays: $line");
+        $lib_ref->milestone("$line");
 
         if ($line =~ /ARRAY (\S+) level=(\w+) num-devices=(\d+)/)
         {
             ($array, $level, $num_devices) = ($1, $2, $3);
             #udevadm sometime return udev symlink instead of physical device, bnc#547580
             $array = $udevmapping->{$array} if $udevmapping->{$array};
-            $logger->milestone("Tools::ReadRAID1Arrays: set array $array level $level and device count to $num_devices");
+            $lib_ref->milestone("set array $array level $level and device count to $num_devices");
         }
         elsif ($level eq "raid1" and $line =~ /devices=(\S+)/)
         {
             # we could test $num_device against number of found devices to
             # detect degradedmode but that does not matter here (really?) 
-             $logger->milestone("Tools::ReadRAID1Arrays: set to array $array values $1");
+             $lib_ref->milestone("set to array $array values $1");
       	     my @devices = split(/,/,$1);
 	           if ($devices[0]=~ m/^.*\d+$/){ #add only non-disc arrays
       	       $mapping{$array} = [ @devices ];
       	     }
         }
     }
-    $logger->milestone("Tools::ReadRAID1Arrays: finish parsing mdadm --detail --verbose --scan:");
+    $lib_ref->milestone("finish parsing mdadm");
     close( MD );
     return \%mapping;
 }
@@ -725,14 +689,34 @@ See InitLibrary function for example.
 
 =cut
 
-sub GetBootloader {
-    my $path = Bootloader::Path::Sysconfig();
-    my $lt = qx{ . $path && echo \$LOADER_TYPE } or
-	die ("GetBootloader(): Cannot determine the loader type");
-    chomp ($lt);
-    return $lt;
+sub GetBootloader
+{
+  my $val = GetSysconfigValue("LOADER_TYPE");
+
+  $lib_ref->milestone("loader = $val") if $lib_ref;
+
+  return $val;
 }   
 
+=item
+C<< $loader = Bootloader::Tools::GetSecureBoot (); >>
+
+returns the secureboot settings. Reads the value from sysconfig.
+Returns the string - secureboot enabled (yes,no) .
+
+See InitLibrary function for example.
+
+=cut
+
+sub GetSecureBoot
+{
+  my $val = GetSysconfigValue("SECURE_BOOT");
+
+  $lib_ref->milestone("secureboot = $val") if $lib_ref;
+
+  return (($val eq "yes") ? 1 : 0);
+}
+
 =item
 C<< $value = Bootloader::Tools::GetSysconfigValue (); >>
 
@@ -743,13 +727,21 @@ See AddSection for example
 
 =cut
 
-sub GetSysconfigValue {
-    my $key = shift;
-    my $file = Bootloader::Path::Sysconfig();
-    return undef if ( qx{ grep -c ^[[:space:]]*$key $file} == 0);
-    my $value = qx{ . $file && echo \$$key } || "";
-    chomp ($value);
-    return $value;
+sub GetSysconfigValue
+{
+  my $key = shift;
+  my $val;
+  local $_;
+
+  if(open my $fh, Bootloader::Path::Sysconfig()) {
+    while(<$fh>) {
+      $val = $1, last if /^\s*$key\s*=\s*"(.*)"\s*$/;
+    }
+
+    close $fh;
+  }
+
+  return $val;
 }
 
 =item
@@ -760,27 +752,30 @@ needed for it to run properly.
 
 =cut
 
-sub InitLibrary {
-    $lib_ref = Bootloader::Library->new ();
-    my $um = GetUdevMapping();
-    my $mp = ReadMountPoints ($um);
-    my $part = ReadPartitions ($um);
-    my $md = ReadRAID1Arrays ($um);
-    my $mpath = GetMultipath ();
+sub InitLibrary
+{
+  $lib_ref = Bootloader::Library->new();
 
-    $lib_ref->SetLoaderType (GetBootloader ());
-    $lib_ref->DefineMountPoints ($mp);
-    $lib_ref->DefinePartitions ($part);
-    $lib_ref->DefineMDArrays ($md);
-    $lib_ref->DefineMultipath ($mpath);
-    $lib_ref->DefineUdevMapping($um);
+  my $um = GetDeviceMapping();
+  my $mp = ReadMountPoints($um);
+  my $part = ReadPartitions($um);
+  my $md = ReadRAID1Arrays($um);
+  my $mpath = GetMultipath();
 
-    # parse Bootloader configuration files   
-    $lib_ref->ReadSettings();
+  $lib_ref->SetLoaderType(GetBootloader());
+  $lib_ref->DefineMountPoints($mp);
+  $lib_ref->DefinePartitions($part);
+  $lib_ref->DefineMDArrays($md);
+  $lib_ref->DefineMultipath($mpath);
+  $lib_ref->DefineUdevMapping($um);
+  $lib_ref->SetSecureBoot (GetSecureBoot ());
 
-    DumpLog ($lib_ref->{"loader"});
+  # parse Bootloader configuration files   
+  $lib_ref->ReadSettings();
 
-    return $lib_ref;
+  $lib_ref->milestone("done");
+
+  return $lib_ref;
 }
 
 
@@ -789,49 +784,47 @@ sub match_section {
     my ($sect_ref, $opt_ref,) = @_;
     my $match = 1;
 
-    my $core_lib = $lib_ref->{"loader"};
-
-    $core_lib->l_milestone ("Tools::match_section: matching section name: " . $sect_ref->{"name"});
+    $lib_ref->milestone("matching section name: " . $sect_ref->{"name"});
 
     foreach my $opt (keys %{$opt_ref}) {
 	next unless exists $sect_ref->{"$opt"};
 	# FIXME: if opt_ref doesn't have (hdX,Y), there is a mountpoint, thus remove it from sect_ref
         # FIXME: to compare !!
-	$core_lib->l_milestone ("Tools::match_section: matching key: $opt");
+	$lib_ref->milestone("matching key: $opt");
 	if ($opt eq "image" or $opt eq "initrd") {
 	    $match = (ResolveCrossDeviceSymlinks($sect_ref->{"$opt"}) eq
 		      ResolveCrossDeviceSymlinks($opt_ref->{"$opt"}));
 	    # Print info for this match
-	    $core_lib->l_milestone ("Tools::match_section: key: $opt, matched: " .
+	    $lib_ref->milestone("key: $opt, matched: " .
 		ResolveCrossDeviceSymlinks($sect_ref->{"$opt"}) .
 		", with: " . ResolveCrossDeviceSymlinks($opt_ref->{"$opt"}) . ", result: $match");
 	}
 	else {
 	    $match = ($sect_ref->{"$opt"} eq $opt_ref->{"$opt"});
 	    # Print info for this match
-	    $core_lib->l_milestone ("Tools::match_section: key: $opt, matched: " .
+	    $lib_ref->milestone("key: $opt, matched: " .
 		$sect_ref->{"$opt"} . ", with: " . $opt_ref->{"$opt"} . ", result: $match");
 	}
 	last unless $match;
     }
-    $core_lib->l_milestone ("Tools::match_section: end result: $match");
+    $lib_ref->milestone("end result: $match");
     return $match;
 }
 
 
 # internal: normalize options in a way needed for 'match_section'
-sub normalize_options {
-    my $opt_ref = shift;
+sub normalize_options
+{
+  my $opt_ref = shift;
+  local $_;
 
-    my $core_lib = $lib_ref->{"loader"};
-
-    foreach ("image", "initrd" ) {
-	# Print found sections to logfile
-	$core_lib->l_milestone ("Tools::normalize_options: key: $_, resolving if exists:" . $opt_ref->{"$_"});
-	$opt_ref->{"$_"} = ResolveCrossDeviceSymlinks($opt_ref->{"$_"})
-	    if exists $opt_ref->{"$_"};
-	$core_lib->l_milestone ("Tools::normalize_options: resolved result:" . $opt_ref->{"$_"});
-    }
+  for ( "image", "initrd" ) {
+    next unless exists $opt_ref->{$_};
+    # print found sections to logfile
+    $lib_ref->milestone("before: '$_' => '$opt_ref->{$_}'");
+    $opt_ref->{$_} = ResolveCrossDeviceSymlinks($opt_ref->{$_});
+    $lib_ref->milestone("after: '$_' => '$opt_ref->{$_}'");
+  }
 }
 
 
@@ -881,12 +874,11 @@ bootloader to use the current configuration
 =cut
 
 
-sub UpdateBootloader {
-    my $avoid_init = shift;
+sub UpdateBootloader
+{
+  my $avoid_init = shift;
 
-    my $ret = $lib_ref->UpdateBootloader ($avoid_init);
-    DumpLog ($lib_ref->{"loader"});
-    return $ret;
+  return $lib_ref->UpdateBootloader($avoid_init);
 }
 
 
@@ -1017,16 +1009,13 @@ sub SetGlobals {
     $glob_ref->{"__modified"} = 1;
     $lib_ref->SetGlobalSettings ($glob_ref);
     unless (defined $lib_ref->WriteSettings (1)) {
-      $lib_ref->l_error ("Cannot write bootloader configuration file.");
-      DumpLog ($lib_ref->{"loader"});
+      $lib_ref->error("Cannot write bootloader configuration file.");
       return undef;
     }
     unless (defined $lib_ref->UpdateBootloader (1)) { # avoid initialization but write config to the right place
-      $lib_ref->l_error ("Cannot write bootloader configuration file.");
-      DumpLog ($lib_ref->{"loader"});
+      $lib_ref->error("Cannot write bootloader configuration file.");
       return undef;
     }
-    DumpLog ($lib_ref->{"loader"});
 }
 
 
@@ -1040,13 +1029,11 @@ sub GetSectionList {
     my %option = @_;
     my $loader = GetBootloader ();
 
-    my $core_lib = $lib_ref->{"loader"};
-
     normalize_options(\%option);
     my @sections = @{$lib_ref->GetSections ()};
 
     # Print sections from file to logfile
-    $core_lib->l_milestone ("Tools::GetSectionList: sections from file:\n' " .
+    $lib_ref->milestone("sections from file:\n' " .
 			join("'\n' ",
 			     map {
 				 $_->{"name"};
@@ -1058,11 +1045,10 @@ sub GetSectionList {
     } @sections;
 
     # Print found sections to logfile
-    $core_lib->l_milestone ("Tools::GetSectionList: Found sections:\n' " .
+    $lib_ref->milestone("Found sections:\n' " .
 			join("'\n' ", @section_names) . "'\n"
 		       );
 
-    DumpLog ($lib_ref->{"loader"});
     return @section_names;
 }
 
@@ -1160,13 +1146,12 @@ sub AddSection {
     my %def = ();
 
     my @sections = @{$lib_ref->GetSections ()};
-    my $core_lib = $lib_ref->{"loader"};
 
     my $fitting_section = GetFittingSection($name,$option{"image"},$option{"type"},\@sections);
 
     if ($fitting_section) {
       %def = %{$fitting_section};
-      $core_lib->l_milestone ("Tools::AddSection: Fitting section found so use it");
+      $lib_ref->milestone("Fitting section found so use it.");
       while ((my $k, my $v) = each (%def)) {
         if (substr ($k, 0, 2) ne "__" && $k ne "original_name"
         		&& $k ne "initrd") {
@@ -1182,7 +1167,7 @@ sub AddSection {
       	}
       }
       } else {
-        $core_lib->l_milestone ("Tools::AddSection: Fitting section not found. Use sysconfig values as fallback.");
+        $lib_ref->milestone("Fitting section not found. Use sysconfig values as fallback.");
         my $sysconf;
         if ($name =~ m/^Failsafe.*$/ or $option{"original_name"} eq "failsafe") {
           $sysconf =  GetSysconfigValue("FAILSAFE_APPEND");
@@ -1235,38 +1220,33 @@ sub AddSection {
     }
 
     $new{"__modified"} = 1;
-    $new{"root"} = $lib_ref->GetMountPoints()->{"/"};
+
+    # if zipl.conf uses only 'parmfile', then don't create extra parameters
+    if(!($fitting_section && $new{parmfile} && !$new{append})) {
+      $new{root} = $lib_ref->GetMountPoints()->{"/"};
+    }
 
     my $match = '';
     my $new_name = '';
 
     my $loader = Bootloader::Tools::GetBootloader ();
 
-    if ($loader ne "grub" and $loader ne "lilo") {
-        # Search for duplicate boot entry label and rename them in a unique way
-	foreach my $s (@sections) {
-	    while ((my $k, my $v) = each (%$s)) {
-		if ($k eq "name" && index ($v, $new{"name"}) >= 0) {
-		    $match += 1;
-		    $new_name = $new{"name"} . "V" . $match;
+    # If we have a bootloader that uses label names to specify the default
+    # entry we need unique labels: so, append '_NUMBER', if necessary.
+    #
+    # cf. Core::FixSectionName()
 
-		    if ($new_name eq $v) {
-			$match += 1;
-			$new_name = $new{"name"} . "V" . $match;
-		    }
-		    $new{"name"} = $new_name;
-		}
-	    }
-	}
+    if ($loader ne "grub" && $loader ne "grub2") {
+        my $max_idx = undef;
+        for my $s (@sections) {
+            $max_idx = $2 + 0 if $s->{name} =~ /^$new{name}(_(\d+))?$/ && $2 >= $max_idx;
+        }
+
+        $new{name} .= "_" . ($max_idx + 1) if defined $max_idx;
     }
 
     # Print new section to be added to logfile
-    $core_lib->l_milestone ("Tools::AddSection: New section to be added :\n\n' " .
-			join("'\n' ",
-			     map {
-				 $_ . " => '" . $new{$_} . "'";
-			     } keys %new) . "'\n"
-		       );
+    $lib_ref->milestone("New section to be added:", \%new);
 
     # Put new entries on top
     unshift @sections, \%new;
@@ -1314,12 +1294,11 @@ sub AddSection {
     }
 
     # Print all available sections to logfile
-    $core_lib->l_milestone (
-	"Tools::AddSection: All available sections (including new ones):\n");
+    $lib_ref->milestone("All available sections (including new ones):");
 
     my $section_count = 1;
     foreach my $s (@sections) {
-	$core_lib->l_milestone ("$section_count. section :\n' " .
+	$lib_ref->milestone("$section_count. section :\n' " .
 			    join("'\n' ",
 				 map {
 				     m/^__/ ? () : $_ . " => '" . $s->{$_} . "'";
@@ -1360,7 +1339,7 @@ sub AddSection {
     }
 
     # Print globals to logfile
-    $core_lib->l_milestone ("Tools::AddSection: Global section of config :\n\n' " .
+    $lib_ref->milestone("Global section of config:\n\n' " .
 			join("'\n' ",
 			     map {
 				 m/^__/ ? () : $_ . " => '" . $glob_ref->{$_} . "'";
@@ -1368,16 +1347,13 @@ sub AddSection {
 		       );
 
     unless (defined $lib_ref->WriteSettings (1)) {
-      $core_lib->l_error ("Cannot write bootloader configuration file.");
-      DumpLog ($lib_ref->{"loader"});
+      $lib_ref->error("Cannot write bootloader configuration file.");
       return undef;
     }
     unless (defined $lib_ref->UpdateBootloader (1)) { # avoid initialization but write config to the right place
-      $core_lib->l_error ("Cannot write bootloader configuration file.");
-      DumpLog ($lib_ref->{"loader"});
+      $lib_ref->error("Cannot write bootloader configuration file.");
       return undef;
     }
-    DumpLog ($lib_ref->{"loader"});
 }
 
 
@@ -1415,12 +1391,10 @@ sub RemoveSections {
     my $default_section = $glob_ref->{"default"} || "";
     my $default_removed = 0;
 
-    my $loader = GetBootloader ();
-
-    my $core_lib = $lib_ref->{"loader"};
+    my $loader = GetBootloader();
 
     # Print section to be removed to logfile
-    $core_lib->l_milestone ("Tools::RemoveSections: Old section to be removed :\n\n' " .
+    $lib_ref->milestone("Old section to be removed :\n\n' " .
 			join("'\n' ",
 			     map {
 				 $_ . " => '" . $option{$_} . "'";
@@ -1428,12 +1402,11 @@ sub RemoveSections {
 		       );
 
     # Print all available sections (before removal) to logfile
-    $core_lib->l_milestone (
-	"Tools::RemoveSections: All available sections (before removal):\n");
+    $lib_ref->milestone("All available sections (before removal):\n");
 
     my $section_count = 1;
     foreach my $s (@sections) {
-	$core_lib->l_milestone ("$section_count. section :\n' " .
+	$lib_ref->milestone("$section_count. section :\n' " .
 			    join("'\n' ",
 				 map {
 				     m/^__/ ? () : $_ . " => '" . $s->{$_} . "'";
@@ -1456,7 +1429,7 @@ sub RemoveSections {
 	    if $match and $default_section eq $_->{"name"};
 	!$match;
     } @sections;
-    $core_lib->l_milestone("default is removed by grep") if $default_removed;
+    $lib_ref->milestone("default is removed by grep") if $default_removed;
 
     # Detect wether we have an entry with an initrd line referring to a non
     # existing initrd file and remove this section respectively.
@@ -1478,8 +1451,8 @@ sub RemoveSections {
 		if (!$other_part and !-f $initrd_name and
 		    ($_->{"type"} eq "image" or $_->{"type"} eq "xen")) {
 		    $match = 0;
-                    $core_lib->l_milestone (
-                	"Tools::RemoveSections: Remove non-existing initrd :".$_->{"name"}." -- $initrd_name \n");
+                    $lib_ref->milestone(
+                	"Remove non-existing initrd :".$_->{"name"}." -- $initrd_name \n");
 		}
 
 		$default_removed = 1
@@ -1502,12 +1475,12 @@ sub RemoveSections {
 	\@section_names_after_removal);
 
     # Print all available sections (after removal) to logfile
-    $core_lib->l_milestone (
-	"Tools::RemoveSections: All available sections (after removal):\n");
+    $lib_ref->milestone(
+	"All available sections (after removal):\n");
 
     $section_count = 1;
     foreach my $s (@sections) {
-	$core_lib->l_milestone ("$section_count. section :\n' " .
+	$lib_ref->milestone("$section_count. section :\n' " .
 			    join("'\n' ",
 				 map {
 				     m/^__/ ? () : $_ . " => '" . $s->{$_} . "'";
@@ -1519,24 +1492,20 @@ sub RemoveSections {
     $lib_ref->SetSections (\@sections);
     if ($default_removed) {
 	$glob_ref->{"default"} = $sections[0]{"name"};
-        $core_lib->l_milestone ( "removed default");
+        $lib_ref->milestone("removed default");
 	$glob_ref->{"removed_default"} = 1;
     }
     $glob_ref->{"__modified"} = 1; # needed because of GRUB - index of default
 				   # may change even if not deleted
     $lib_ref->SetGlobalSettings ($glob_ref);
     unless (defined $lib_ref->WriteSettings (1)) {
-      $core_lib->l_error ("Cannot write bootloader configuration file.");
-      DumpLog ($lib_ref->{"loader"});
+      $lib_ref->error("Cannot write bootloader configuration file.");
       return undef;
     }
     unless (defined $lib_ref->UpdateBootloader (1)) { # avoid initialization but write config to the right place
-      $core_lib->l_error ("Cannot write bootloader configuration file.");
-      DumpLog ($lib_ref->{"loader"});
+      $lib_ref->error("Cannot write bootloader configuration file.");
       return undef;
     }
-
-    DumpLog ($lib_ref->{"loader"});
 }
 
 
