@@ -13,6 +13,8 @@ BEGIN {
         swig_vname_set => ["function", "void", "any", "string"],
         swig_vdevice_get => ["function", "string", "any"],
         swig_vdevice_set => ["function", "void", "any", "string"],
+        swig_num_get => ["function", "integer", "any"],
+        swig_num_set => ["function", "void", "any", "integer"],
     );
 }
 
