@@ -19,6 +19,10 @@ BEGIN {
         swig_chunkSizeK_set => ["function", "void", "any", "integer"],
         swig_devices_get => ["function", "string", "any"],
         swig_devices_set => ["function", "void", "any", "string"],
+        swig_spares_get => ["function", "string", "any"],
+        swig_spares_set => ["function", "void", "any", "string"],
+        swig_inactive_get => ["function", "boolean", "any"],
+        swig_inactive_set => ["function", "void", "any", "boolean"],
     );
 }
 
